"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Bootstrap = void 0;

const tslib_1 = require("tslib");

const assert_1 = tslib_1.__importDefault(require("assert"));

const path_1 = require("path");

const fs = tslib_1.__importStar(require("fs-extra"));

const lodash_1 = require("lodash");

const esbuild = tslib_1.__importStar(require("esbuild"));

const core_1 = require("./core");

class Bootstrap {
  constructor() {
    this.debounceRun = lodash_1.debounce(() => {
      this.run();
    }, 1500);
    this.server = new core_1.Service({
      paths: {
        cwd: process.cwd()
      }
    });
  }

  getConfig() {
    assert_1.default(fs.existsSync(this.userConfigFilePath), '请在根目录上创建配置文件');
    const configTemporaryFilePath = this.server.resolveTmpPath(`config.js`);
    esbuild.buildSync({
      entryPoints: [this.userConfigFilePath],
      bundle: true,
      platform: 'node',
      treeShaking: true,
      outfile: configTemporaryFilePath
    });
    delete require.cache[require.resolve(configTemporaryFilePath)];
    return require(configTemporaryFilePath).default;
  }

  init() {
    const cwd = process.cwd();
    this.userConfigPath = path_1.join(cwd, 'config');
    this.userConfigFilePath = path_1.join(this.userConfigPath, `index.ts`);
    const userConfig = this.getConfig();
    this.server.setEnv(userConfig.env || process.env.NODE_ENV || 'development').setUserConfig(userConfig);
    return this.server.plugin.initPlugins();
  }

  getServer() {
    return this.server;
  }

  async run(isReCompile = true) {
    this.server.logger.printCompiling();

    if (isReCompile) {
      this.server.setUserConfig(this.getConfig());
    }

    await this.server.run(isReCompile);
    this.server.logger.stopAndPersistOk('生成配置完成');
    setTimeout(() => {
      this.server.logger.printWhenFirstDone();
    });
  }

}

exports.Bootstrap = Bootstrap;