"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

const tslib_1 = require("tslib");

const cloneDeep_1 = tslib_1.__importDefault(require("lodash/cloneDeep"));

const SEPARATOR = '^^^';
const EMPTY_PATH = '_';

const routeToChunkName = componentPath => componentPath.replace(/^.(\/|\\)/, '').replace(/(\/|\\)/g, '__').replace(/\.jsx?$/, '').replace(/\.tsx?$/, '').replace(/^src__/, '').replace(/\.\.__/g, '').replace(/[\[\]]/g, '').replace(/^pages__/, 'p__').replace(/^page__/, 'p__');

const routeToImportName = componentPath => componentPath.replace(/\//g, '').replace(/@/g, '').replace(/\./g, '').toLocaleUpperCase();

const pMinDelayImportName = 'pMinDelay';

function default_1(config) {
  var _config$dynamicImport;

  const clonedRoutes = cloneDeep_1.default(config.routes);
  let routeImportNameTemplate = '';
  patchRoutes(clonedRoutes);

  if ((_config$dynamicImport = config.dynamicImport) === null || _config$dynamicImport === void 0 ? void 0 : _config$dynamicImport.delay) {
    routeImportNameTemplate += `\nimport ${pMinDelayImportName} from 'p-min-delay';`;
  }

  function patchRoutes(routes) {
    routes.forEach(patchRoute);
  }

  function patchRoute(route) {
    if (route.component && !isFunctionComponent(route.component)) {
      const webpackChunkName = routeToChunkName(route.component);
      const routeImportName = routeToImportName(route.component);

      if (!config.dynamicImport) {
        routeImportNameTemplate += `\nimport ${routeImportName} from '${route.component}';`;
      }

      route.component = [route.component, webpackChunkName, routeImportName, route.path || EMPTY_PATH].join(SEPARATOR);
    }

    if (route.routes) {
      patchRoutes(route.routes);
    }
  }

  function isFunctionComponent(component) {
    return /^\((.+)?\)(\s+)?=>/.test(component) || /^function([^\(]+)?\(([^\)]+)?\)([^{]+)?{/.test(component);
  }

  function getLoadingOptionText() {
    var _config$dynamicImport2;

    return !((_config$dynamicImport2 = config.dynamicImport) === null || _config$dynamicImport2 === void 0 ? void 0 : _config$dynamicImport2.loading) ? '' : `, { fallback: <LoadingComponent />, }`;
  }

  function getWebpackPreloadText() {
    var _config$dynamicImport3;

    return !((_config$dynamicImport3 = config.dynamicImport) === null || _config$dynamicImport3 === void 0 ? void 0 : _config$dynamicImport3.webpackPreload) ? '' : `/* webpackPrefetch: true */`;
  }

  function getWebpackChunkNameText(webpackChunkName) {
    return `/* webpackChunkName: '${webpackChunkName}' */`;
  }

  function replacer(key, value) {
    var _config$dynamicImport4;

    switch (key) {
      case 'component':
        if (isFunctionComponent(value)) return value;
        const [component, webpackChunkName, routeImportName] = value.split(SEPARATOR);

        if (!config.dynamicImport) {
          return `${routeImportName}`;
        }

        const webpackText = [getWebpackChunkNameText(webpackChunkName), getWebpackPreloadText()].filter(Boolean).join(' ');
        let importText = `import(${webpackText}'${component}')`;

        if ((_config$dynamicImport4 = config.dynamicImport) === null || _config$dynamicImport4 === void 0 ? void 0 : _config$dynamicImport4.delay) {
          importText = `${pMinDelayImportName}(${importText}, ${config.dynamicImport.delay})`;
        }

        return `dynamic( () => ${importText}${getLoadingOptionText()})`;

      case 'wrappers':
        const wrappers = value.map(wrapper => {
          if (config.dynamicImport) {
            let loading = '';

            if (config.dynamicImport.loading) {
              loading = `, loading: LoadingComponent`;
            }

            return `dynamic({ loader: () => import(/* webpackChunkName: 'wrappers' */'${wrapper}')${loading}})`;
          }

          return `require('${wrapper}').default`;
        });
        return `[${wrappers.join(', ')}]`;

      default:
        return value;
    }
  }

  const routes = JSON.stringify(clonedRoutes, replacer, 2).replace(/\"component\": (\"(.+?)\")/g, (global, m1, m2) => {
    return `"component": ${m2.replace(/\^/g, '"')}`;
  }).replace(/\"wrappers\": (\"(.+?)\")/g, (global, m1, m2) => {
    return `"wrappers": ${m2.replace(/\^/g, '"')}`;
  }).replace(/\\r\\n/g, '\r\n').replace(/\\n/g, '\r\n');
  return {
    importNameTemplate: routeImportNameTemplate,
    routes
  };
}

exports.default = default_1;